"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("../common");
const system_1 = require("../system");
const Utils_1 = require("../system/Utils");
function entityUri(domain, module, text) {
    const domainUri = common_1.domainToUri(domain);
    const moduleUri = common_1.moduleToUri(module);
    if (text) {
        return `${domainUri}/${moduleUri}/entity/${text}`;
    }
    return `${domainUri}/${moduleUri}/entity`;
}
function externalEntityUri(moduleName, text) {
    const moduleNameUri = common_1.toLowerCamel(moduleName);
    if (text) {
        return `core/entity/external/${moduleNameUri}/${text}`;
    }
    return `core/entity/external/${moduleNameUri}`;
}
function baseGet(api, domain, module, id) {
    const uri = entityUri(domain, module, id);
    return api.get(uri);
}
function baseCreate(api, domain, module, data) {
    const uri = entityUri(domain, module, 'create');
    return api.post(uri, data);
}
function baseDelete(api, domain, module, id, data) {
    const uri = entityUri(domain, module, 'delete');
    return api.post(uri, Object.assign(Object.assign({}, data), { id }));
}
function baseView(api, domain, module, id, data) {
    const uri = entityUri(domain, module, 'view');
    return api.post(uri, Object.assign(Object.assign({}, data), { id }));
}
function entity(domain, module) {
    const entityDomain = domain;
    const entityModule = module;
    common_1.validateDomainAndModule(entityDomain, entityModule);
    const api = system_1.default.getLibModule(system_1.LibModule.API);
    function get(id) {
        return baseGet(api, entityDomain, entityModule, id);
    }
    function create(data) {
        return baseCreate(api, entityDomain, entityModule, data);
    }
    function del(id, data) {
        return baseDelete(api, entityDomain, entityModule, id, data);
    }
    function view(id, data) {
        return baseView(api, entityDomain, entityModule, id, data);
    }
    return Object.freeze({
        get,
        create,
        delete: del,
        view,
    });
}
const addCustomAttributeUri = 'core/entity/external/customAttribute/add';
const removeCustomAttributeUri = 'core/entity/external/customAttribute/remove';
const updateCustomAttributeUri = 'core/entity/external/customAttribute/modify';
function external(moduleName) {
    const api = system_1.default.getLibModule(system_1.LibModule.API);
    const createUri = externalEntityUri(moduleName, 'create');
    const updateUri = externalEntityUri(moduleName, 'update');
    const searchUri = 'search';
    function get(id) {
        const uri = externalEntityUri(moduleName, id);
        return api.get(uri);
    }
    function create(data) {
        return api.post(createUri, data);
    }
    function update(data) {
        return api.post(updateUri, data);
    }
    function search(filters, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield api.put(searchUri, Object.assign(Object.assign({}, options), { filters, module_name: moduleName }));
            return response ? response : [];
        });
    }
    function addCustomAttribute(moduleId, attribute) {
        const payload = {
            module: { module_name: moduleName, module_id: moduleId },
            custom_attribute: attribute,
        };
        return api.post(addCustomAttributeUri, payload);
    }
    function removeCustomAttribute(moduleId, attributeId) {
        const payload = {
            module: { module_name: moduleName, module_id: moduleId },
            custom_attribute_id: attributeId,
        };
        return api.post(removeCustomAttributeUri, payload);
    }
    function updateCustomAttribute(moduleId, attribute) {
        const payload = {
            module: { module_name: moduleName, module_id: moduleId },
            custom_attribute: attribute,
        };
        return api.post(updateCustomAttributeUri, payload);
    }
    return Object.freeze({
        get,
        create,
        update,
        search,
        addCustomAttribute,
        removeCustomAttribute,
        updateCustomAttribute,
    });
}
function internalGet(domain, module, id) {
    common_1.validateDomainAndModule(domain, module);
    return baseGet(system_1.default.getLibModule(system_1.LibModule.API), domain, module, id);
}
function internalCreate(domain, module, data) {
    common_1.validateDomainAndModule(domain, module);
    return baseCreate(system_1.default.getLibModule(system_1.LibModule.API), domain, module, data);
}
function internalDelete(domain, module, id, data) {
    common_1.validateDomainAndModule(domain, module);
    return baseDelete(system_1.default.getLibModule(system_1.LibModule.API), domain, module, id, data);
}
function internalView(domain, module, id, data) {
    common_1.validateDomainAndModule(domain, module);
    return baseView(system_1.default.getLibModule(system_1.LibModule.API), domain, module, id, data);
}
const moduleInstance = Object.freeze(Utils_1.makeCallable(entity, {
    get: internalGet,
    create: internalCreate,
    delete: internalDelete,
    view: internalView,
    external,
}));
exports.default = system_1.default.sealModule(moduleInstance);
